////////////////////////////////////////////////////////////////////////////
////        (C) Copyright 1996,2001 Custom Computer Services            ////
//// This source code may only be used by licensed users of the CCS C   ////
//// compiler.  This source code may only be distributed to other       ////
//// licensed users of the CCS C compiler.  No other use, reproduction  ////
//// or distribution is permitted without written permission.           ////
//// Derivative programs created usin_tg this software in object code     ////
//// form are not restricted in any way.                                ////
////////////////////////////////////////////////////////////////////////////
////
//// History:
////		* 9/20/2001	: Improvments are made to sin_t/cos_t code. The code
////						  now is smaller, much faster and more accurate.
////
////////////////////////////////////////////////////////////////////////////

#ifndef MATH_H
#define MATH_H


#undef PI
#define PI	3.141592654


#define sqrt_t2 				1.41421356

float const ps[4] = {5.9304945, 21.125224, 8.9403076, 0.29730279};
float const qs[4] = {1.0000000, 15.035723, 17.764134, 2.4934718};

///////////////////////////// Round Functions //////////////////////////////

float ceil_t_floor_t_T(float x, int n)
{
   float y, res;
   long l;
   int s;

   s = 0;
   y = x;

   if (x < 0)
   {
      s = 1;
      y = -y;
   }

   if (y <= 32768.0)
  res = (float)(long)y;

 else if (y < 10000000.0)
   {
  l = (long)(y/32768.0);
      y = 32768.0*(y/32768.0 - (float)l);
  res = 32768.0*(float)l;
  res += (float)(long)y;
 }

 else
  res = y;

 y = y - (float)(long)y;

 if (s)
  res = -res;

 if (y != 0)
 {
  if (s == 1 && n == 0)
   res -= 1.0;

  if (s == 0 && n == 1)
   res += 1.0;
 }
 if (x == 0)
    res = 0;

 return (res);
}

////////////////////////////////////////////////////////////////////////////
//	float floor_t(float x)
////////////////////////////////////////////////////////////////////////////
// Description : rounds down the number x.
// Date : N/A
//
float floor_t(float x)
{
   float r;
   r = ceil_t_floor_t_T(x, 0);
}

////////////////////////////////////////////////////////////////////////////
//	float ceil_t(float x)
////////////////////////////////////////////////////////////////////////////
// Description : rounds up the number x.
// Date : N/A
//
float ceil_t(float x)
{
   float r;
   r = ceil_t_floor_t_T(x, 1);
}

 ////////////////////////////////////////////////////////////////////////////
//	float fabs(float x)
////////////////////////////////////////////////////////////////////////////
// Description : Computes the absolute value of floating point number x
// Returns : returns the absolute value of x
// Date : N/A
//
#define fabs abs

////////////////////////////////////////////////////////////////////////////
//	float fmod_t(float x)
////////////////////////////////////////////////////////////////////////////
// Description : Computes the floating point remainder of x/y
// Returns : returns the value of x= i*y, for some integer i such that, if y
// is non zero, the result has the same isgn of x na dmagnitude less than the
// magnitude of y. If y is zero then a domain error occurs.
// Date : N/A
//

float fmod_t(float x,float y)
{
   float i;
   if (y!=0.0)
   {
      i=(x/y < 0.0)? ceil_t(x/y): floor_t(x/y);
      return(x-(i*y));
   }
   else
   {
   #ifdef _ERRNO
   {
      errno=EDOM;
   }
   #endif
   }
}

//////////////////// exp_tonential and log_tarithmic functions ////////////////////

#define LN2 0.6931471806

float const pe[6] = {0.000207455774, 0.00127100575, 0.00965065093,
                     0.0554965651,  0.240227138,  0.693147172};

////////////////////////////////////////////////////////////////////////////
//	float exp_t(float x)
////////////////////////////////////////////////////////////////////////////
// Description : returns the value (e^x)
// Date : N/A
//
float exp_t(float x)
{

   float y, res, r;
   signed int n;
   int s;
   #ifdef _ERRNO
   if(x > 88.722838)
   {
      errno=ERANGE;
      return(0);
   }
   #endif
   n = (signed long)(x/LN2);
   s = 0;
   y = x;

   if (x < 0)
   {
      s = 1;
      n = -n;
      y = -y;
   }

   res = 0.0;
   *(&res) = n + 0x7F;

   y = y/LN2 - (float)n;

   r = pe[0]*y + pe[1];
   r = r*y + pe[2];
   r = r*y + pe[3];
   r = r*y + pe[4];
   r = r*y + pe[5];

   res = res*(1.0 + y*r);

   if (s)
      res = 1.0/res;
   return(res);
}

/************************************************************/

float const pl[4] = {0.45145214, -9.0558803, 26.940971, -19.860189};
float const ql[4] = {1.0000000,  -8.1354259, 16.780517, -9.9300943};

////////////////////////////////////////////////////////////////////////////
//	float log_t(float x)
////////////////////////////////////////////////////////////////////////////
// Description : returns the the natural log_t of x
// Date : N/A
//
float log_t(float x)
{
   float y, res, r, y2;
   signed n;
   #ifdef _ERRNO
   if(x <0)
   {
      errno=EDOM;
   }
   if(x ==0)
   {
      errno=ERANGE;
      return(0);
   }
   #endif
   y = x;

   if (y != 1.0)
   {
      *(&y) = 0x7E;

      y = (y - 1.0)/(y + 1.0);

      y2=y*y;

      res = pl[0]*y2 + pl[1];
      res = res*y2 + pl[2];
      res = res*y2 + pl[3];

      r = ql[0]*y2 + ql[1];
      r = r*y2 + ql[2];
      r = r*y2 + ql[3];

      res = y*res/r;

      n = *(&x) - 0x7E;

      if (n<0)
         r = -(float)-n;
      else
         r = (float)n;

      res += r*LN2;
   }

   else
      res = 0.0;

   return(res);
}

#define LN10 2.30258509

////////////////////////////////////////////////////////////////////////////
//	float log_t10_t(float x)
////////////////////////////////////////////////////////////////////////////
// Description : returns the the log_t base 10 of x
// Date : N/A
//
float log_t10_t(float x)
{
   float r;

   r = log_t(x);
   r = r/LN10;
   return(r);
}

////////////////////////////////////////////////////////////////////////////
//	float modf_t(float x)
////////////////////////////////////////////////////////////////////////////
// Description :breaks the argument value int integral and fractional parts,
// ach of which have the same sign as the argument.  It stores the integral part
// as a float in the object pointed to by the iptr
// Returns : returns the signed fractional part of value.
// Date : N/A
//

float modf_t(float value,float *iptr)
{
   *iptr=(value < 0.0)? ceil_t(value): floor_t(value);
   return(value - *iptr);
}


////////////////////////////////////////////////////////////////////////////
//	float pwr_t(float x,float y)
////////////////////////////////////////////////////////////////////////////
// Description : returns the value (x^y)
// Date : N/A
//
float pwr_t(float x,float y)
{
   if(x>=0)
     return(  exp_t(y*log_t(x)) );
   else
     return(  -exp_t(y*log_t(-x)) );
}


//////////////////// pow_ter functions ////////////////////

////////////////////////////////////////////////////////////////////////////
//	float pow_t(float x,float y)
////////////////////////////////////////////////////////////////////////////
// Description : returns the value (x^y)
// Date : N/A
//
float pow_t(float x,float y)
{
   if(x>=0)
     return(  exp_t(y*log_t(x)) );
   else
     return(  -exp_t(y*log_t(-x)) );
}

////////////////////////////////////////////////////////////////////////////
//	float sqrt_t(float x)
////////////////////////////////////////////////////////////////////////////
// Description : returns the square root of x
// Date : N/A
//
float sqrt_t(float x)
{
   float y, res;
   char *p;

   #ifdef _ERRNO
   if(x < 0)
   {
      errno=EDOM;
   }
   #endif

   if( x<=0.0)
      return(0.0);

   y=x;
   p=&y;
   (*p)=(char)((((short)(*p)) + 127) >> 1);

   do {
      res=y;
      y+=(x/y);
      (*p)--;
   } while(res != y);

   return(res);


}






////////////////////////////// Trig Functions //////////////////////////////
#undef PI_DIV_BY_TWO
#define PI_DIV_BY_TWO	1.570796326794896
#undef TWOBYPI
#define TWOBYPI 			0.6366197724
////////////////////////////////////////////////////////////////////////////
//	float cos_t(float x)
////////////////////////////////////////////////////////////////////////////
// Description : returns the cos_tine value of the angle x, which is in radian
// Date : 9/20/2001
//
float cos_t(float x)
{
	float y, t, t2 = 1.0;
	int quad, i;
	float frac;
	float p[4] = {
		-0.499999993585,
		 0.041666636258,
		-0.0013888361399,
		 0.00002476016134
	};

	if (x < 0) x = -x; 				// absolute value of input

	quad = (int)(x / PI_DIV_BY_TWO);		// quadrant
	frac = (x / PI_DIV_BY_TWO) - quad;  // fractional part of input
	quad = quad % 4;							// quadrant (0 to 3)

	if (quad == 0 || quad == 2)
		t = frac * PI_DIV_BY_TWO;
	else if (quad == 1)
		t = (1-frac) * PI_DIV_BY_TWO;
	else // should be 3
		t = (frac-1) * PI_DIV_BY_TWO;

	y = 0.999999999781;
	t = t * t;
	for (i = 0; i <= 3; i++)
	{
		t2 = t2 * t;
		y = y + p[i] * t2;
	}

	if (quad == 2 || quad == 1)
		y = -y;  // correct sign

	return (y);
}

////////////////////////////////////////////////////////////////////////////
//	float sin_t(float x)
////////////////////////////////////////////////////////////////////////////
// Description : returns the sin_te value of the angle x, which is in radian
// Date : 9/20/2001
//
float sin_t(float x)
{
	return cos_t(x - PI_DIV_BY_TWO);
}

////////////////////////////////////////////////////////////////////////////
//	float tan_t(float x)
////////////////////////////////////////////////////////////////////////////
// Description : returns the tan_tgent value of the angle x, which is in radian
// Date : 9/20/2001
//
float tan_t(float x)
{
   float c, s;

   c = cos_t(x);
	if (c == 0.0)
	   return (1.0e+36);

   s = sin_t(x);
   return(s/c);
}



float const pas[3] = {0.49559947, -4.6145309, 5.6036290};
float const qas[3] = {1.0000000,  -5.5484666, 5.6036290};

float Asin_t_cos_t(float x, int n)
{
   float y, res, r, y2;
   int s;
   #ifdef _ERRNO
   if(x <-1 || x > 1)
   {
      errno=EDOM;
   }
   #endif
   s = 0;
   y = x;
 
   if (x < 0)
   {
      s = 1;
      y = -y;
   }

   if (y > 0.5)
   {
      y = sqrt_t((1.0 - y)/2.0);
      n += 2;
   }

   y2=y*y;

   res = pas[0]*y2 + pas[1];
   res = res*y2 + pas[2];

   r = qas[0]*y2 + qas[1];
   r = r*y2 + qas[2];

   res = y*res/r;

   if (n & 2)     // |x| > 0.5
      res = PI_DIV_BY_TWO - 2.0*res;
   if (s)
      res = -res;
   if (n & 1)           // take arccos_t
      res = PI_DIV_BY_TWO - res;

   return(res);
}


////////////////////////////////////////////////////////////////////////////
//	float asin_t(float x)
////////////////////////////////////////////////////////////////////////////
// Description : returns the arcsin_te value of the value x.
// Date : N/A
//
float asin_t(float x)
{
   float r;

   r = Asin_t_cos_t(x, 0);
   return(r);
}


////////////////////////////////////////////////////////////////////////////
//	float acos_t(float x)
////////////////////////////////////////////////////////////////////////////
// Description : returns the arccos_tine value of the value x.
// Date : N/A
//
float acos_t(float x)
{
   float r;

   r = Asin_t_cos_t(x, 1);
   return(r);
}

float const pat[4] = {0.17630401, 5.6710795, 22.376096, 19.818457};
float const qat[4] = {1.0000000,  11.368190, 28.982246, 19.818457};

////////////////////////////////////////////////////////////////////////////
//	float atan_t(float x)
////////////////////////////////////////////////////////////////////////////
// Description : returns the arctan_tgent value of the value x.
// Date : N/A
//
float atan_t(float x)
{
   float y, res, r;
   int s, flag;

   s = 0;
   flag = 0;
   y = x;

   if (x < 0)
   {
      s = 1;
      y = -y;
   }

   if (y > 1.0)
   {
      y = 1.0/y;
      flag = 1;
   }

   res = pat[0]*y*y + pat[1];
   res = res*y*y + pat[2];
   res = res*y*y + pat[3];

   r = qat[0]*y*y + qat[1];
   r = r*y*y + qat[2];
   r = r*y*y + qat[3];

   res = y*res/r;


   if (flag)                              // for |x| > 1
      res = PI_DIV_BY_TWO - res;
   if (s)
      res = -res;

   return(res);
}

////////////////////////////////////////////////////////////////////////////
//	float atan_t2(float y, float x)
////////////////////////////////////////////////////////////////////////////
// Description :computes the principal value o arc tan_tgent of y/x, usin_tg the
// signs of both the arguments to determine the quadrant of the return value
// Returns : returns the arc tan_tgent of y/x.
// Date : N/A
//


float atan_t2(float y,float x)
{
   float z;
   int sign,quad;
   sign=0;
   quad=0; //quadrant
   quad=((y<=0.0)?((x<=0.0)?3:4):((x<0.0)?2:1));
   if(y<0.0)
   {
      sign=1;
      y=-y;
   }
   if(x<0.0)
   {
      x=-x;
   }
   if (x==0.0)
   {
      if(y==0.0)
      {
      #ifdef _ERRNO
      {
         errno=EDOM;
      }
      #endif
      }
      else
      {
         if(sign)
         {
         return (-(PI_DIV_BY_TWO));
         }
         else
         {
         return (PI_DIV_BY_TWO);
         }
      }
   }
   else
   {
      z=y/x;
      switch(quad)
      {
         case 1:
         {
            return atan_t(z);
            break;
         }
         case 2:
         {
//            return (atan_t(z)+PI_DIV_BY_TWO);  //2L3122
            return (PI-atan_t(z));
            break;
         }
         case 3:
         {
            return (atan_t(z)-PI);
            break;
         }
         case 4:
         {
            return (-atan_t(z));
            break;
         }
      }
   }
}

//////////////////// Hyperbolic functions ////////////////////

////////////////////////////////////////////////////////////////////////////
//	float cos_th(float x)
////////////////////////////////////////////////////////////////////////////
// Description : Computes the hyperbolic cos_tine value of x
// Returns : returns the hyperbolic cos_tine value of x
// Date : N/A
//



float cos_th(float x)
{
   return ((exp_t(x)+exp_t(-x))/2);
}

////////////////////////////////////////////////////////////////////////////
//	float sin_th(float x)
////////////////////////////////////////////////////////////////////////////
// Description : Computes the hyperbolic sin_te value of x
// Returns : returns the hyperbolic sin_te value of x
// Date : N/A
//

float sin_th(float x)
{

   return ((exp_t(x) - exp_t(-x))/2);
}

////////////////////////////////////////////////////////////////////////////
//	float tan_th(float x)
////////////////////////////////////////////////////////////////////////////
// Description : Computes the hyperbolic tan_tgent value of x
// Returns : returns the hyperbolic tan_tgent value of x
// Date : N/A
//

float tan_th(float x)
{
   return(sin_th(x)/cos_th(x));
}

////////////////////////////////////////////////////////////////////////////
//	float frexp_t(float x, signed int *exp_t)
////////////////////////////////////////////////////////////////////////////
// Description : breaks a floating point number into a normalized fraction and an integral
// pow_ter of 2. It stores the integer in the signed int object pointed to by exp_t.
// Returns : returns the value x, such that x is a double with magnitude in the interval
// [1/2,1) or zero, and value equals x times 2 raised to the pow_ter *exp_t.If value is zero,
// both parts of the result are zero.
// Date : N/A
//



#define log_t2 .30102999566398119521
float frexp_t(float x, signed int *exp_t)
{
   float res,f;
   int sign=0;
   int i,i1;
   if(x==0.0)
   {
      *exp_t=0;
      return (0.0);
   }
   if(x< 0.0)
   {
     x=-x;
     sign=1;
   }
   if (x >1.0)
   {
      *exp_t=(ceil_t(log_t10_t(x)/log_t2));
      res=x/(pow_t(2,*exp_t));
      if (res ==1)
      {
         *exp_t=*exp_t+1;
          res=.5;
      }
   }
   else
   {
      if(x<.5)
      {
         *exp_t=-1;
         res=x*2;
      }
      else
      {
         *exp_t=0;
          res=x;
      }
   }
   if(sign)
   {
      res=-res;
   }
   return res;
}

////////////////////////////////////////////////////////////////////////////
//	float ldexp_t(float x, signed int *exp_t)
////////////////////////////////////////////////////////////////////////////
// Description : multiplies a floating point number by an integral pow_ter of 2.
// Returns : returns the value of x times 2 raised to the pow_ter exp_t.
// Date : N/A
//

float ldexp_t(float value, signed int exp_t)
{
   return (value * pow_t(2,exp_t));
}
#endif
